#! /bin/sh
#
# Macronix patch release 1.00
# by ITO Takayuki <yuki@fsinet.or.jp>
# http://hp.vector.co.jp/authors/VA001240/beos/index.en.html
#
# This script modifies BeOS R5 Tulip driver
# for use with Macronix MX9871x/9872x chipset
#
# 1.00	first release
#
# PCI vendor IDs:
# DEC      1011h
# Macronix 10D9h
# Liteon   11ADh
#
# PCI device IDs:
# 21040    0002h
# 21041    0014h
# 21140    0009h
# 21142    0019h
# MX98713  0512h
# MX98715  0531h
# MX98725  0531h
# MX98723  0532h
# MX98727  0532h

SRCDIR=/system/add-ons/kernel/drivers
DSTDIR=$HOME/config/add-ons/kernel/drivers
SRCFILE=tulip
DSTFILE=macronix

cd `dirname $0`

alert --warning "This script will apply patch to the Tulip driver for use with Macronix Ethernet chipset." OK Quit > /dev/null
if [ $? -eq 1 ]
then
	alert --stop "Aborted."
	exit 1
fi

if [ ! -f ./bsubst ]
then
	if [ -f bsubst.c ]
	then
		cc -o bsubst bsubst.c
	else
		alert --stop "bsubst is missing."
		exit 1
	fi
fi

# check original driver

if [ -f $DSTDIR/bin/$DSTFILE ]
then
	alert --stop "Macronix patch is already applied."
	exit 1
fi
if [ ! -f $SRCDIR/bin/$SRCFILE ]
then
	alert --stop "Original driver $SRCDIR/bin/$SRCFILE not present."
	exit 1
fi

# chipset detection

TMPFILE=/tmp/poke.log

poke > $TMPFILE <<END
pci
quit
END

ID=`grep 'vendor 10d9' $TMPFILE | head -1 | cut -d ' ' -f 10`
rm $TMPFILE
if [ "x$ID" = x ]
then
	alert --stop "No Macronix chipset detected."
	exit 1
fi

case $ID in
0512)	CHIP=MX98713;;
0531)	CHIP=MX98715/98725;;
0532)	CHIP=MX98723/98727;;
*)	CHIP=unknown;;
esac
if [ $CHIP = unknown ]
then
	alert --info "Unknown Macronix chipset detected. Device ID is $ID."
else
	alert --info "Macronix Ethernet chipset $CHIP detected. Device ID is $ID."
fi

# apply patch to driver

if [ -f $SRCDIR/bin/$SRCFILE.gz ]
then
	alert --stop "Patch is already applied."
	exit 1
fi

alert --info "Processing $SRCDIR/bin/$SRCFILE."
< $SRCDIR/bin/$SRCFILE \
./bsubst "11 10 02 00" "d9 10 12 05" | \
./bsubst "11 10 14 00" "d9 10 31 05" | \
./bsubst "11 10 09 00" "d9 10 32 05" | \
./bsubst "11 10 19 00 74 19" "d9 10 00 00 74 19" | \
./bsubst "11 10 00 00 02 00" "d9 10 00 00 12 05" | \
./bsubst "11 10 00 00 14 00" "d9 10 00 00 31 05" | \
./bsubst "11 10 00 00 09 00" "d9 10 00 00 32 05" | \
./bsubst "11 10 00 00 19 00" "d9 10 00 00 00 00" \
> $DSTDIR/bin/$DSTFILE

cmp -s $SRCDIR/bin/$SRCFILE $DSTDIR/bin/$DSTFILE
if [ $? -eq 0 ]
then
	alert --stop "Patch is not applicable. Nothing altered."
	rm $DSTDIR/bin/$DSTFILE
	exit 1
fi

gzip $SRCDIR/bin/$SRCFILE
[ -e $DSTDIR/dev/net/$DSTFILE ] || ln -s ../../bin/$DSTFILE $DSTDIR/dev/net/$DSTFILE

alert --info "Done."
exit 0
